"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tailPipeline = exports.togglePausePipeline = exports.detachPipeline = exports.attachPipeline = exports.buildPipeline = exports.processResponse = exports.initPipelines = void 0;
const sp_common_1 = require("@streamdal/protos/protos/sp_common");
const kv_js_1 = require("./kv.js");
const register_js_1 = require("./register.js");
const tokenBucket_js_1 = require("./utils/tokenBucket.js");
const wasm_js_1 = require("./wasm.js");
const initPipelines = async (configs) => {
    try {
        if (register_js_1.internal.pipelineInitialized) {
            return;
        }
        console.debug("initializing pipelines");
        const { response } = await configs.grpcClient.getAttachCommandsByService({
            serviceName: configs.serviceName.toLowerCase(),
        }, { meta: { "auth-token": configs.streamdalToken } });
        for await (const [k, v] of Object.entries(response.wasmModules)) {
            await (0, wasm_js_1.instantiateWasm)(k, v.bytes);
        }
        for await (const command of response.active) {
            await (0, exports.processResponse)(command);
        }
        register_js_1.internal.pipelineInitialized = true;
    }
    catch (e) {
        console.error("Error initializing pipelines", e);
    }
};
exports.initPipelines = initPipelines;
const processResponse = async (response) => {
    if (response.command.oneofKind === "kv") {
        (0, kv_js_1.kvCommand)(response.command.kv);
        return;
    }
    if (!response.audience) {
        response.command.oneofKind !== "keepAlive" &&
            console.debug("command response has no audience, ignoring");
        return;
    }
    switch (response.command.oneofKind) {
        case "attachPipeline":
            response.command.attachPipeline.pipeline &&
                (await (0, exports.attachPipeline)(response.audience, response.command.attachPipeline.pipeline));
            break;
        case "detachPipeline":
            (0, exports.detachPipeline)(response.audience, response.command.detachPipeline.pipelineId);
            break;
        case "pausePipeline":
            (0, exports.togglePausePipeline)(response.audience, response.command.pausePipeline.pipelineId, true);
            break;
        case "resumePipeline":
            (0, exports.togglePausePipeline)(response.audience, response.command.resumePipeline.pipelineId, false);
            break;
        case "tail":
            (0, exports.tailPipeline)(response.audience, response.command.tail);
            break;
    }
};
exports.processResponse = processResponse;
const buildPipeline = async (pipeline) => {
    return {
        ...pipeline,
        steps: await Promise.all(pipeline.steps.map(async (step) => {
            await (0, wasm_js_1.instantiateWasm)(step.WasmId, step.WasmBytes);
            return {
                ...step,
                WasmBytes: undefined,
            };
        })),
    };
};
exports.buildPipeline = buildPipeline;
const attachPipeline = async (audience, pipeline) => {
    const key = (0, register_js_1.audienceKey)(audience);
    const existing = register_js_1.internal.pipelines.get(key);
    const built = await (0, exports.buildPipeline)(pipeline);
    register_js_1.internal.pipelines.set(key, existing
        ? existing.set(pipeline.id, built)
        : new Map([[pipeline.id, built]]));
};
exports.attachPipeline = attachPipeline;
const detachPipeline = (audience, pipelineId) => register_js_1.internal.pipelines.get((0, register_js_1.audienceKey)(audience))?.delete(pipelineId);
exports.detachPipeline = detachPipeline;
const togglePausePipeline = (audience, pipelineId, paused) => {
    const key = (0, register_js_1.audienceKey)(audience);
    const existing = register_js_1.internal.pipelines.get(key);
    const p = existing?.get(pipelineId);
    existing &&
        p &&
        register_js_1.internal.pipelines.set(key, existing.set(p.id, { ...p, paused }));
};
exports.togglePausePipeline = togglePausePipeline;
const tailPipeline = (audience, { request }) => {
    console.debug("received a tail command for audience", audience);
    if (!request) {
        console.debug("no tail reqeuest details specified, skipping");
        return;
    }
    switch (request.type) {
        case sp_common_1.TailRequestType.START: {
            console.debug("received a START tail: adding entry to audiences for tail id", audience);
            // Create inner map if it doesn't exist
            if (!register_js_1.internal.audiences.has((0, register_js_1.audienceKey)(audience))) {
                register_js_1.internal.audiences.set((0, register_js_1.audienceKey)(audience), {
                    audience,
                    tails: new Map(),
                });
            }
            // Add entry (@JH, OK if overwritten?)
            request.id &&
                register_js_1.internal.audiences.get((0, register_js_1.audienceKey)(audience))?.tails.set(request.id, {
                    tail: request.type === sp_common_1.TailRequestType.START,
                    tailRequestId: request.id,
                    sampleBucket: new tokenBucket_js_1.TokenBucket(request.sampleOptions?.sampleRate, request.sampleOptions?.sampleIntervalSeconds),
                });
            break;
        }
        case sp_common_1.TailRequestType.STOP: {
            console.debug("received a STOP tail: removing entry from audiences for tail id", request.id);
            request.id &&
                register_js_1.internal.audiences.get((0, register_js_1.audienceKey)(audience))?.tails.delete(request.id);
            break;
        }
        default:
            console.error("unknown tail request type ", request.type);
            break;
    }
};
exports.tailPipeline = tailPipeline;
//# sourceMappingURL=pipeline.js.map