"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runStep = exports.resultCondition = exports.processPipelines = exports.processPipeline = exports.retryProcessPipelines = exports.MAX_PAYLOAD_SIZE = void 0;
const sp_pipeline_1 = require("@streamdal/protos/protos/sp_pipeline");
const sp_sdk_1 = require("@streamdal/protos/protos/sp_sdk");
const sp_wsm_1 = require("@streamdal/protos/protos/sp_wsm");
const audience_js_1 = require("./audience.js");
const metrics_js_1 = require("./metrics.js");
const pipeline_js_1 = require("./pipeline.js");
const register_js_1 = require("./register.js");
const schema_js_1 = require("./schema.js");
const tail_js_1 = require("./tail.js");
const wasm_js_1 = require("./wasm.js");
exports.MAX_PAYLOAD_SIZE = 1024 * 1024; // 1 megabyte
const MAX_PIPELINE_RETRIES = 10;
const PIPELINE_RETRY_INTERVAL = 1000;
const retryProcessPipelines = async ({ configs, audience, data, }) => {
    let retries = 1;
    try {
        if (register_js_1.internal.registered) {
            return (0, exports.processPipelines)({
                configs,
                audience,
                data,
            });
        }
        console.info(`not yet registered with the grpc server, retrying process pipeline in ${PIPELINE_RETRY_INTERVAL / 1000} seconds...`);
        return new Promise((resolve) => {
            const intervalId = setInterval(() => {
                retries++;
                if (MAX_PIPELINE_RETRIES && retries >= MAX_PIPELINE_RETRIES) {
                    clearInterval(intervalId);
                    return;
                }
                if (register_js_1.internal.registered) {
                    console.debug(`retrying process pipeline...`);
                    clearInterval(intervalId);
                    return resolve((0, exports.processPipelines)({
                        configs,
                        audience,
                        data,
                    }));
                }
                console.debug(`retrying process pipeline in ${PIPELINE_RETRY_INTERVAL / 1000} seconds; ${retries} of ${MAX_PIPELINE_RETRIES} retries`);
            }, PIPELINE_RETRY_INTERVAL);
        });
    }
    catch (e) {
        console.error("Error running process pipeline", e);
    }
    const errorMessage = "Node SDK not registered with the server, skipping pipeline. Is the server running?";
    console.error(errorMessage);
    return Promise.resolve({
        data,
        dropMessage: false,
        pipelineStatus: [],
        error: true,
        errorMessage,
    });
};
exports.retryProcessPipelines = retryProcessPipelines;
const processPipeline = ({ originalData, audience, configs, pipeline, }) => {
    const pipelineStatus = {
        id: pipeline.id,
        name: pipeline.name,
        stepStatus: [],
    };
    let data = originalData;
    for (const step of pipeline.steps) {
        if (configs.dryRun) {
            console.debug(`Dry run set. Found pipeline: ${pipeline.name}, step: ${step.name}...not running.`);
            continue;
        }
        console.debug(`running pipeline: ${pipeline.name}, step: ${step.name}...`);
        const { data: newData, stepStatus } = (0, exports.runStep)({
            originalData,
            audience,
            configs,
            step,
            pipeline,
        });
        data = newData;
        pipelineStatus.stepStatus = [...pipelineStatus.stepStatus, stepStatus];
        if ([sp_sdk_1.AbortStatus.CURRENT, sp_sdk_1.AbortStatus.ALL, sp_sdk_1.AbortStatus.DROP_MESSAGE].includes(stepStatus.abortStatus)) {
            break;
        }
    }
    return { data, pipelineStatus };
};
exports.processPipeline = processPipeline;
const processPipelines = async ({ configs, audience, data, }) => {
    await (0, pipeline_js_1.initPipelines)(configs);
    await (0, audience_js_1.addAudience)({ configs: configs, audience });
    const key = (0, register_js_1.audienceKey)(audience);
    const pipelines = register_js_1.internal.pipelines.get(key);
    void (0, metrics_js_1.audienceMetrics)(audience, data.length);
    if (!pipelines) {
        const errorMessage = "no active pipelines found for this audience, returning data";
        console.debug(errorMessage);
        (0, tail_js_1.sendTail)({
            configs,
            audience,
            originalData: data,
        });
        return {
            data,
            error: true,
            dropMessage: false,
            errorMessage,
            pipelineStatus: [],
        };
    }
    const response = {
        data,
        pipelineStatus: [],
        error: false,
        dropMessage: false,
        errorMessage: "",
    };
    for (const pipeline of pipelines.values()) {
        const { data, pipelineStatus } = (0, exports.processPipeline)({
            originalData: response.data,
            audience,
            configs,
            pipeline,
        });
        response.data = data;
        response.pipelineStatus = [...response.pipelineStatus, pipelineStatus];
        const lastStatus = pipelineStatus.stepStatus.at(-1)?.abortStatus;
        if (lastStatus &&
            [sp_sdk_1.AbortStatus.ALL, sp_sdk_1.AbortStatus.DROP_MESSAGE].includes(lastStatus)) {
            break;
        }
    }
    (0, tail_js_1.sendTail)({
        configs,
        audience,
        originalData: data,
        newData: response.data,
    });
    const finalStatus = response.pipelineStatus.at(-1)?.stepStatus.at(-1);
    return Promise.resolve({
        ...response,
        dropMessage: finalStatus?.abortStatus === sp_sdk_1.AbortStatus.DROP_MESSAGE,
        error: !!finalStatus?.error,
        errorMessage: finalStatus?.errorMessage ?? "",
    });
};
exports.processPipelines = processPipelines;
const notifyStep = async (configs, step, pipeline) => {
    console.debug("notifying error step", step.name);
    try {
        await configs.grpcClient.notify({
            pipelineId: pipeline.id,
            stepName: step.name,
            occurredAtUnixTsUtc: Date.now().toString(),
        }, { meta: { "auth-token": configs.streamdalToken } });
    }
    catch (e) {
        console.error("error sending notification to server", e);
    }
};
/**
 * DROP_MESSAGE is the same as ABORT_ALL, but sets a flag on the final response,
 * and overrides the other settings. ABORT_ALL overrides ABORT_CURRENT,
 */
const getStepStatus = (resultConditions) => resultConditions.includes(sp_pipeline_1.PipelineStepCondition.DISCARD_MESSAGE)
    ? sp_sdk_1.AbortStatus.DROP_MESSAGE
    : resultConditions.includes(sp_pipeline_1.PipelineStepCondition.ABORT_ALL)
        ? sp_sdk_1.AbortStatus.ALL
        : resultConditions.includes(sp_pipeline_1.PipelineStepCondition.ABORT_CURRENT)
            ? sp_sdk_1.AbortStatus.CURRENT
            : sp_sdk_1.AbortStatus.UNSET;
const resultCondition = ({ configs, step, pipeline, stepStatus, }) => {
    const conditions = stepStatus.error ? step.onFailure : step.onSuccess;
    conditions.includes(sp_pipeline_1.PipelineStepCondition.NOTIFY) &&
        void notifyStep(configs, step, pipeline);
    stepStatus.abortStatus = getStepStatus(conditions);
};
exports.resultCondition = resultCondition;
const runStep = ({ originalData, audience, configs, step, pipeline, }) => {
    const stepStatus = {
        name: step.name,
        error: false,
        errorMessage: "",
        abortStatus: sp_sdk_1.AbortStatus.UNSET,
    };
    const payloadSize = originalData.length;
    let data = originalData;
    try {
        const { outputPayload, outputStep, exitCode, exitMsg } = (0, wasm_js_1.runWasm)({
            step,
            originalData,
        });
        //
        // output gets passed back as data for the next function
        const error = exitCode !== sp_wsm_1.WASMExitCode.WASM_EXIT_CODE_SUCCESS;
        data = error ? originalData : outputPayload;
        stepStatus.error = error;
        stepStatus.errorMessage = error ? exitMsg : "";
        !error &&
            step.name === "Infer Schema" &&
            void (0, schema_js_1.sendSchema)({ configs, audience, schema: outputStep });
    }
    catch (error) {
        console.error(`error running pipeline step - ${step.name}`, error);
        stepStatus.error = true;
        stepStatus.errorMessage = error.toString();
    }
    (0, exports.resultCondition)({ configs, step, pipeline, stepStatus });
    void (0, metrics_js_1.stepMetrics)({ audience, stepStatus, pipeline, payloadSize });
    return { data, stepStatus };
};
exports.runStep = runStep;
//# sourceMappingURL=process.js.map