import { WASMExitCode, WASMRequest, WASMResponse, } from "@streamdal/protos/protos/sp_wsm";
// eslint-disable-next-line import/no-unresolved
import { WASI } from "wasi";
import { kvExists } from "./kv.js";
import { MAX_PAYLOAD_SIZE } from "./process.js";
import { internal } from "./register.js";
const [nodeVersionMajor] = process.versions.node.split(".");
const wasi = new WASI({
    ...(Number(nodeVersionMajor) >= 20 ? { version: "preview1" } : {}),
    preopens: {
        "/sandbox": "./",
    },
});
export const instantiateWasm = async (wasmId, wasmBytes) => {
    if (!wasmId || !wasmBytes || wasmBytes.length === 0) {
        console.debug("Wasm info missing, skipping instantiation, .");
        return;
    }
    if (internal.wasmModules.has(wasmId)) {
        console.debug("Wasm exists, skipping instantiation");
        return;
    }
    const wasm = await WebAssembly.compile(wasmBytes);
    const instantiated = await WebAssembly.instantiate(wasm, {
        wasi_snapshot_preview1: wasi.wasiImport,
        env: {
            kvExists: (pointer, length) => kvExists(instantiated.exports, pointer, length),
        },
    });
    internal.wasmModules.set(wasmId, instantiated);
};
const readResponse = (pointer, buffer) => {
    //
    // Shift right by 32 bits to get the start value
    const start = Number(pointer >> BigInt(32));
    //
    // Bitwise AND operation with 0xFFFFFFFF to get the length
    const length = Number(pointer & BigInt(0xffffffff));
    return buffer.slice(start, start + length);
};
export const writeResponse = (pointer, length) => {
    //
    // Left shift the pointer value by 32 bits
    const start = BigInt(pointer) << BigInt(32);
    //
    // Combine the shifted start and length using bitwise OR
    return start | BigInt(length);
};
export const runWasm = ({ step, originalData, }) => {
    if (originalData.length > MAX_PAYLOAD_SIZE) {
        return {
            outputStep: null,
            outputPayload: new Uint8Array(),
            exitCode: WASMExitCode.WASM_EXIT_CODE_INTERNAL_ERROR,
            exitMsg: "Payload exceeds maximum size",
        };
    }
    const request = WASMRequest.create({
        step: {
            name: step.name,
            onSuccess: step.onSuccess,
            onFailure: step.onFailure,
            step: step.step,
        },
        inputPayload: originalData,
    });
    const { exports } = internal.wasmModules.get(step.WasmId);
    const { memory, alloc, [step.WasmFunction]: f } = exports;
    const requestBytes = WASMRequest.toBinary(request);
    const ptr = alloc(requestBytes.length);
    const mem = new Uint8Array(memory.buffer, ptr, requestBytes.length);
    mem.set(requestBytes);
    const returnPtr = BigInt(f(ptr, requestBytes.length));
    const response = readResponse(returnPtr, new Uint8Array(memory.buffer));
    return WASMResponse.fromBinary(response);
};
//# sourceMappingURL=wasm.js.map